<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	if (!defined('_PS_VERSION_'))
		exit;	
		
	require_once("PolarisLinkException.php");
	require_once("PolarisLinkSite.php");	
	
	/*
	 * Classe de base pour gérer les caches
	 */
	class PolarisLinkCacheManager
	{
		private $_CachesMap = array();
		
		public function __construct()
		{			
		}
		
		public function have($type)
		{
			return isset($this->_CachesMap[$type]) && $this->_CachesMap[$type] != null;
		}
		
		public function get($type)
		{
			if (isset($this->_CachesMap[$type]))
				return $this->_CachesMap[$type];
			return NULL;
		}
		
		public function add(PolarisLinkCache $cache)
		{
			$this->_CachesMap[$cache->getName()] = $cache;
		}
	} 
	
	/*
	 * Classe de base pour gérer un cache
	 */
	class PolarisLinkCache
	{
		private $_CacheMap = array();
		private $_Name = NULL;
		private $_ObjToCreate = NULL;
		private $_Loaded = false;
		private $_SQLLoad = NULL;		
		
		public function getName()
		{
			return $this->_Name;
		}
		
		public function isLoaded()
		{
			return $this->_Loaded;
		}
		
		public function __construct($name, $sqlToLoad = NULL, $sqlToSave = NULL, $objToCreate = NULL)
		{
			$this->_Name = $name;
			$this->_SQLLoad = $sqlToLoad;
			$this->_ObjToCreate = $objToCreate;
		}		
		
		public function reset()
		{
			$this->_CacheMap = array();			
			$this->_Loaded = false;
		}
		
		public function load()
		{
			if (!empty($this->_SQLLoad))
			{
				$db = Db::getInstance();
				$results = $db->executeS($this->_SQLLoad, true, false);
				if ($results === false)
					throw new PrestaShopDatabaseException($db->getMsgError());

				if ($this->_ObjToCreate == NULL)
				{
					foreach ($results as $item)
						$this->_CacheMap[$item["k"]] = $item["v"];
				}
				else					
				{
					$class = $this->_ObjToCreate;					
					foreach ($results as $item)
						$this->_CacheMap[$item["k"]] = new $class($item["v"]);
				}
			}
			$this->_Loaded = true;
		}
		
		public function listKeys()
		{
			if (!$this->_Loaded)
				$this->load();
			
			return array_keys($this->_CacheMap);
		}
		
		public function listValues()
		{
			if (!$this->_Loaded)
				$this->load();
			
			return array_values($this->_CacheMap);
		}
		
		public function set($key, $value)
		{			
			$this->_CacheMap[$key] = $value;
		}
		
		public function getKey($searchedValue)
		{
			if (!$this->_Loaded)
				$this->load();
				
			foreach ($this->_CacheMap as $key => $value)
			{
				if ($value == $searchedValue)
					return $key;
			}
			
			return NULL;
		}
		
		public function get($key)
		{
			if (isset($this->_CacheMap[$key]))
				return $this->_CacheMap[$key];
			
			// pas trouvé ? on charge depuis la base
			if (!$this->_Loaded)
			{
				$this->load();
				
				// On réessai
				if (isset($this->_CacheMap[$key]))
					return $this->_CacheMap[$key];
			}
			
			// toujours pas trouvé ?
			return NULL;
		}
	} 
	
	/*
	 * Classe de base pour gérer un cache
	 */
	class PolarisLinkExtRefCache extends PolarisLinkCache
	{
		private $_Table;
		private $_LinkID;
		
		public function __construct($linkID, $table)
		{
			$this->_Table = $table;
			$this->_LinkID = $linkID;
			parent::__construct("pr_".$this->_Table,
					"SELECT e.id_polaris as k, e.id_ps AS v FROM "._DB_PREFIX_."polarislink_ref_".$this->_Table." AS e WHERE ".sprintf("e.id_link = %s", $this->_LinkID));
		}
		
		public function set($key, $value)
		{
			$currentValue = $this->get($key);
			if ($currentValue != $value)
			{
				if ($currentValue == NULL)
					Db::getInstance()->query(sprintf("INSERT INTO "._DB_PREFIX_."polarislink_ref_".$this->_Table." (id_link, id_polaris, id_ps) VALUES (%s, '%s', %s)",  $this->_LinkID, pSQL($key), $value));
				else if (empty($value))
					Db::getInstance()->query(sprintf("DELETE FROM "._DB_PREFIX_."polarislink_ref_".$this->_Table." WHERE id_link = %s AND id_polaris = '%s'",  $this->_LinkID, pSQL($key)));
				else
					Db::getInstance()->query(sprintf("UPDATE "._DB_PREFIX_."polarislink_ref_".$this->_Table." SET id_ps = %s WHERE id_link = %s AND id_polaris = '%s')", $value, $this->_LinkID, pSQL($key)));
				
				parent::set($key, $value);
			}
		}
	}
