<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	if (!defined('_PS_VERSION_'))
		exit;
			
	require_once( _PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkAPI.php");
	require_once( _PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkJobManager.php");
	
	/*
	 * exécution des travaux
	 *
	 */
	class PolarisLinkAPI_migrateModuleFrontController extends PolarisLinkAPI
	{
		protected function getObjects($className, $tableName, $idField, $sinceId, $limit = 100)
		{
			$objects = array();

			$sql = sprintf("SELECT ".$idField." FROM "._DB_PREFIX_.$tableName." WHERE ".$idField." > %d ORDER BY ".$idField." ASC LIMIT %d", $sinceId, $limit);
            foreach (Db::getInstance()->executeS(
                $sql, 
                true, false) as $row)
			{
                $obj = new $className($row[$idField]);

                $objects[] = $obj;
            }

            return $objects;
		}
		
		protected function runAPI()
		{
			// Surtout plus de cache !!!
			Db::getInstance()->disableCache();

			$secure_token = isset($_REQUEST['secure_token']) && !empty($_REQUEST['secure_token']) ? trim($_REQUEST['secure_token']) : "";
			
			$config = require _PS_CACHE_DIR_ . 'appParameters.php';
			$must_have = sha1("PolarisLinkMigrateChallenge".date("Y-m-d").$config['parameters']['secret']);

			if ($secure_token != $must_have)
			{
				header("HTTP/1.1 403 Access denied");  
				echo "Access denied";
				die();
			}

			$sinceId = isset($_REQUEST['sinceId']) && !empty($_REQUEST['sinceId']) ? trim($_REQUEST['sinceId']) : 0;
            $limit = isset($_REQUEST['limit']) && !empty($_REQUEST['limit']) ? trim($_REQUEST['limit']) : 100;		

            header("Content-Type: application/json; charset=utf-8");

			switch (isset($_REQUEST["item"]) ? $_REQUEST["item"] : "")
			{
				case "customers":
					echo json_encode($this->getObjects("Customer", "customer", "id_customer", $sinceId, $limit));
					break;
				case "addresses":
					echo json_encode($this->getObjects("Address", "address", "id_address", $sinceId, $limit));
					break;
				case "vouchers":
					echo json_encode($this->getObjects("CartRule", "cart_rule", "id_cart_rule", $sinceId, $limit));
					break;
				case "messages":
					echo json_encode($this->getObjects("Message", "message", "id_message", $sinceId, $limit));
					break;
				case "customerThreads":
					echo json_encode($this->getObjects("CustomerThread", "customer_thread", "id_customer_thread", $sinceId, $limit));
					break;
				case "customerMessages":
					echo json_encode($this->getObjects("CustomerMessage", "customer_message", "id_customer_message", $sinceId, $limit));
					break;
				case "orders":
					echo json_encode($this->getObjects("Order", "orders", "id_order", $sinceId, $limit));
					break;
				case "orderDetails":
					echo json_encode($this->getObjects("OrderDetail", "order_detail", "id_order_detail", $sinceId, $limit));
					break;
				case "orderInvoice":
					echo json_encode($this->getObjects("OrderInvoice", "order_invoice", "id_order_invoice", $sinceId, $limit));
					break;
				case "orderHistory":
					echo json_encode($this->getObjects("OrderHistory", "order_history", "id_order_history", $sinceId, $limit));
					break;
				case "orderReturn":
					echo json_encode($this->getObjects("OrderReturn", "order_return", "id_order_return", $sinceId, $limit));
					break;
				case "orderSlip":
					echo json_encode($this->getObjects("OrderSlip", "order_slip", "id_order_slip", $sinceId, $limit));
					break;
				case "orderPayment":
					echo json_encode($this->getObjects("OrderPayment", "order_payment", "id_order_payment", $sinceId, $limit));
					break;
				default:
					echo json_encode([]);
			}

			return true;
		}
	}
 